@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004 Brian Tarricone, <brian@tarricone.org>
  *  Copyright (c) 2010 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,13 +29,12 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-
-#include <X11/Xlib.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include <stdarg.h>
 
 #define XFDESKTOP_CHANNEL        "xfce4-desktop"
-#define DEFAULT_BACKDROP         DATADIR "/backgrounds/xfce/xfce-teal.jpg"
+#define DEFAULT_BACKDROP         DATADIR "/backgrounds/xfce/xfce-shapes.svg"
 #define DEFAULT_ICON_FONT_SIZE   12
 #define DEFAULT_ICON_SIZE        48
 #define ITHEME_FLAGS             (GTK_ICON_LOOKUP_USE_BUILTIN)
@@ -64,7 +63,8 @@
 #define DESKTOP_ICONS_SHOW_FILESYSTEM        "/desktop-icons/file-icons/show-filesystem"
 #define DESKTOP_ICONS_SHOW_REMOVABLE         "/desktop-icons/file-icons/show-removable"
 
-#define DESKTOP_MENU_MAX_TEMPLATE_FILES     "/desktop-menu/max-template-files"
+#define DESKTOP_MENU_MAX_TEMPLATE_FILES      "/desktop-menu/max-template-files"
+#define DESKTOP_MENU_DELETE                  "/desktop-menu/show-delete"
 
 /**
  * File information namespaces queried for #GFileInfo objects.
@@ -108,6 +108,31 @@ GtkWidget* xfdesktop_menu_create_menu_item_with_markup(const gchar *name,
 GtkWidget* xfdesktop_menu_create_menu_item_with_mnemonic(const gchar *name,
                                                          GtkWidget   *image);
 
+void xfdesktop_get_screen_dimensions(GdkScreen *gcreen,
+                                     gint      *width,
+                                     gint      *height);
+
+gint xfdesktop_get_monitor_num(GdkDisplay *display,
+                               GdkMonitor *monitor);
+
+gint xfdesktop_get_current_monitor_num(GdkDisplay *display);
+
+gboolean xfdesktop_workspace_get_number_and_total(XfwWorkspaceManager *workspace_manager,
+                                                  XfwWorkspace *workspace,
+                                                  gint *workspace_number,
+                                                  gint *total_workspace_count);
+
+void xfdesktop_tree_path_free(gpointer data);
+
+void xfdesktop_widget_unrealize(GtkWidget *widget,
+                                gpointer data);
+
+void xfdesktop_object_ref(gpointer data,
+                          gpointer user_data);
+
+void xfdesktop_object_unref(gpointer data,
+                            GClosure *closure);
+
 #if defined(G_HAVE_ISO_VARARGS)
 
 #define XF_DEBUG(...) xfdesktop_debug (__func__, __FILE__, __LINE__, __VA_ARGS__)
