@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004-2008 Brian J. Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2008 Brian J. Tarricone <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,9 +44,10 @@
 #endif
 
 #ifdef ENABLE_DESKTOP_MENU
-static gboolean show_delete_option = TRUE;
+static gboolean inited = FALSE;
 static gboolean show_desktop_menu = TRUE;
 static gboolean show_desktop_menu_icons = TRUE;
+static XfconfChannel *xfconf_channel = NULL;
 static GarconMenu *garcon_menu = NULL;
 #endif
 
@@ -70,9 +71,11 @@ menu_populate(GtkMenuShell *menu, gint scale_factor)
     if(garcon_menu == NULL) {
         garcon_menu = garcon_menu_new_applications();
     }
-    desktop_menu = garcon_gtk_menu_new (garcon_menu);
+    desktop_menu = g_object_new(GARCON_GTK_TYPE_MENU,
+                                "show-menu-icons", show_desktop_menu_icons,
+                                "menu", garcon_menu,
+                                NULL);
     XF_DEBUG("show desktop menu icons %s", show_desktop_menu_icons ? "TRUE" : "FALSE");
-    garcon_gtk_menu_set_show_menu_icons(GARCON_GTK_MENU(desktop_menu), show_desktop_menu_icons);
 
     /* check to see if the menu is empty.  if not, add the desktop menu
     * to a submenu */
@@ -116,6 +119,9 @@ menu_settings_changed(XfconfChannel *channel,
         show_desktop_menu = G_VALUE_TYPE(value)
                             ? g_value_get_boolean(value)
                             : TRUE;
+        if (!show_desktop_menu) {
+            g_clear_object(&garcon_menu);
+        }
     } else if(!strcmp(property, "/desktop-menu/show-icons")) {
         show_desktop_menu_icons = G_VALUE_TYPE(value)
                                   ? g_value_get_boolean(value)
@@ -128,9 +134,7 @@ void
 menu_init(XfconfChannel *channel)
 {
 #ifdef ENABLE_DESKTOP_MENU
-    if(channel) {
-        show_delete_option = xfconf_channel_get_bool(channel, DESKTOP_MENU_DELETE, TRUE);
-    }
+    g_return_if_fail(!inited);
 
     if(!channel || xfconf_channel_get_bool(channel, "/desktop-menu/show", TRUE))
     {
@@ -147,11 +151,28 @@ menu_init(XfconfChannel *channel)
     if(channel) {
         g_signal_connect(G_OBJECT(channel), "property-changed",
                          G_CALLBACK(menu_settings_changed), NULL);
+        xfconf_channel = g_object_ref(channel);
     }
+
+    inited = TRUE;
 #endif
 }
 
 void
 menu_cleanup(void)
 {
+#ifdef ENABLE_DESKTOP_MENU
+    g_return_if_fail(inited);
+
+    if (xfconf_channel != NULL) {
+        g_signal_handlers_disconnect_by_func(xfconf_channel,
+                                             G_CALLBACK(menu_settings_changed),
+                                             NULL);
+        g_clear_object(&xfconf_channel);
+    }
+
+    g_clear_object(&garcon_menu);
+
+    inited = FALSE;
+#endif
 }
