@@ -32,8 +32,8 @@
 G_BEGIN_DECLS
 
 void menu_init(XfconfChannel *channel);
-GtkMenu *menu_populate(GtkMenu *menu, gint scale_factor);
-void menu_cleanup(XfconfChannel *channel);
+GtkMenuShell *menu_populate(GtkMenuShell *menu, gint scale_factor);
+void menu_cleanup(void);
 
 G_END_DECLS
 
