@@ -25,14 +25,15 @@
 #ifndef _XFDESKTOP__WINDOWLIST_H_
 #define _XFDESKTOP__WINDOWLIST_H_
 
+#include "xfce-desktop.h"
+
 #include <xfconf/xfconf.h>
-#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
 void windowlist_init(XfconfChannel *channel);
-GtkMenu *windowlist_populate(GtkMenu *menu, gint scale_factor);
-void windowlist_cleanup(XfconfChannel *channel);
+GtkMenuShell *windowlist_populate(GtkMenuShell *menu, gint scale_factor);
+void windowlist_cleanup(void);
 
 G_END_DECLS
 
