@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004-2007 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2007 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -57,15 +57,16 @@
 #include <time.h>
 #endif
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
 #include <glib.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
+#ifdef ENABLE_X11
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <gdk/gdkx.h>
 #include <cairo-xlib.h>
+#endif  /* ENABLE_X11 */
 
 #ifdef ENABLE_DESKTOP_ICONS
 #include "xfdesktop-icon-view.h"
@@ -78,10 +79,12 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include <xfconf/xfconf.h>
-#include <libwnck/libwnck.h>
 
+#include "menu.h"
+#include "windowlist.h"
 #include "xfdesktop-common.h"
 #include "xfce-desktop.h"
 #include "xfce-desktop-enum-types.h"
@@ -90,10 +93,13 @@
 /* disable setting the x background for bug 7442 */
 //#define DISABLE_FOR_BUG7442
 
+typedef GtkMenuShell *(*PopulateMenuFunc)(GtkMenuShell *, gint);
+
 struct _XfceDesktopPrivate
 {
     GdkScreen *gscreen;
-    WnckScreen *wnck_screen;
+    XfwScreen *xfw_screen;
+    XfwWorkspaceManager *workspace_manager;
     gboolean updates_frozen;
 
     XfconfChannel *channel;
@@ -113,6 +119,8 @@ struct _XfceDesktopPrivate
 
     guint32 grab_time;
 
+    GtkWidget *active_root_menu;
+
 #ifdef ENABLE_DESKTOP_ICONS
     XfceDesktopIconStyle icons_style;
     gboolean icons_font_size_set;
@@ -128,13 +136,6 @@ struct _XfceDesktopPrivate
     gchar *last_filename;
 };
 
-enum
-{
-    SIG_POPULATE_ROOT_MENU = 0,
-    SIG_POPULATE_SECONDARY_ROOT_MENU,
-    N_SIGNALS
-};
-
 enum
 {
     PROP_0 = 0,
@@ -188,8 +189,6 @@ static void hidden_state_changed_cb(GObject *object, XfceDesktop *desktop);
 #endif
 
 
-static guint signals[N_SIGNALS] = { 0, };
-
 /* private functions */
 
 #ifdef ENABLE_DESKTOP_ICONS
@@ -288,29 +287,32 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
 }
 #endif
 
+#ifdef ENABLE_X11
 static void
 set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
                           gint monitor)
 {
-    GdkDisplay *display;
-    gchar property_name[128];
+    if (xfw_windowing_get() == XFW_WINDOWING_X11) {
+        GdkDisplay *display;
+        gchar property_name[128];
 
-    display = gdk_screen_get_display(desktop->priv->gscreen);
-    gdk_x11_display_error_trap_push(display);
+        display = gdk_screen_get_display(desktop->priv->gscreen);
+        xfw_windowing_error_trap_push(display);
 
-    g_snprintf(property_name, 128, XFDESKTOP_IMAGE_FILE_FMT, monitor);
-    if(filename) {
-        gdk_property_change(gdk_screen_get_root_window(desktop->priv->gscreen),
-                            gdk_atom_intern(property_name, FALSE),
-                            gdk_x11_xatom_to_atom(XA_STRING), 8,
-                            GDK_PROP_MODE_REPLACE,
-                            (guchar *)filename, strlen(filename)+1);
-    } else {
-        gdk_property_delete(gdk_screen_get_root_window(desktop->priv->gscreen),
-                            gdk_atom_intern(property_name, FALSE));
+        g_snprintf(property_name, 128, XFDESKTOP_IMAGE_FILE_FMT, monitor);
+        if(filename) {
+            gdk_property_change(gdk_screen_get_root_window(desktop->priv->gscreen),
+                                gdk_atom_intern(property_name, FALSE),
+                                gdk_x11_xatom_to_atom(XA_STRING), 8,
+                                GDK_PROP_MODE_REPLACE,
+                                (guchar *)filename, strlen(filename)+1);
+        } else {
+            gdk_property_delete(gdk_screen_get_root_window(desktop->priv->gscreen),
+                                gdk_atom_intern(property_name, FALSE));
+        }
+
+        xfw_windowing_error_trap_pop_ignored(display);
     }
-
-    gdk_x11_display_error_trap_pop_ignored(display);
 }
 
 static void
@@ -327,7 +329,7 @@ set_real_root_window_surface(GdkScreen *gscreen,
     pixmap_id = cairo_xlib_surface_get_drawable (surface);
 
     display = gdk_screen_get_display(gscreen);
-    gdk_x11_display_error_trap_push(display);
+    xfw_windowing_error_trap_push(display);
 
     /* set root property for transparent Eterms */
     gdk_property_change(groot,
@@ -342,9 +344,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     cairo_pattern_destroy(pattern);
     /* there really should be a standard for this crap... */
 
-    gdk_x11_display_error_trap_pop_ignored(display);
+    xfw_windowing_error_trap_pop_ignored(display);
 #endif
 }
+#endif  /* ENABLE_X11 */
 
 static cairo_surface_t *
 create_bg_surface(GdkScreen *gscreen, gpointer user_data)
@@ -440,7 +443,7 @@ set_accountsservice_user_bg(const gchar *background)
     if (variant != NULL) {
         g_variant_unref (variant);
     } else {
-        g_warning ("Failed to set the background '%s': %s", background, error->message);
+        g_warning ("Failed to register the newly set background with AccountsService '%s': %s", background, error->message);
         g_clear_error (&error);
     }
 
@@ -452,15 +455,14 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     cairo_surface_t *surface = desktop->priv->bg_surface;
+    cairo_surface_t *pix_surface;
     GdkScreen *gscreen = desktop->priv->gscreen;
     GdkDisplay *display;
     gchar *new_filename = NULL;
     GdkRectangle rect;
     cairo_region_t *clip_region = NULL;
     gint i, monitor = -1, current_workspace;
-#ifdef G_ENABLE_DEBUG
-    gchar *monitor_name = NULL;
-#endif
+    gint scale_factor;
 
     TRACE("entering");
 
@@ -502,11 +504,8 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     }
 
 #ifdef G_ENABLE_DEBUG
-    monitor_name = gdk_screen_get_monitor_plug_name(gscreen, monitor);
-
-    XF_DEBUG("backdrop changed for workspace %d, monitor %d (%s)", current_workspace, monitor, monitor_name);
-
-    g_free(monitor_name);
+    XF_DEBUG("backdrop changed for workspace %d, monitor %d (%s)", current_workspace, monitor,
+             gdk_monitor_get_model(gdk_display_get_monitor(display, monitor)));
 #endif
 
     if(xfce_desktop_get_n_monitors(desktop) > 1
@@ -539,7 +538,8 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
                  rect.x, rect.y, rect.width, rect.height);
     }
 
-    xfce_backdrop_set_size(backdrop, rect.width, rect.height);
+    scale_factor = gtk_widget_get_scale_factor(GTK_WIDGET(desktop));
+    xfce_backdrop_set_size(backdrop, rect.width * scale_factor, rect.height * scale_factor);
 
     if(monitor > 0
        && !xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[current_workspace])) {
@@ -608,7 +608,11 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         }
 
         cr = cairo_create(surface);
-        gdk_cairo_set_source_pixbuf(cr, pix, rect.x, rect.y);
+        pix_surface = gdk_cairo_surface_create_from_pixbuf(pix,
+                                                           scale_factor,
+                                                           gtk_widget_get_window(GTK_WIDGET(desktop)));
+        cairo_set_source_surface(cr, pix_surface, rect.x, rect.y);
+        cairo_surface_destroy(pix_surface);
 
         /* clip the area so we don't draw over a previous wallpaper */
         if(clip_region != NULL) {
@@ -622,12 +626,14 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         gtk_widget_queue_draw_area(GTK_WIDGET(desktop), rect.x, rect.y,
                                    rect.width, rect.height);
 
+#ifdef ENABLE_X11
         set_imgfile_root_property(desktop,
                                   xfce_backdrop_get_image_filename(backdrop),
                                   monitor);
 
         /* do this again so apps watching the root win notice the update */
         set_real_root_window_surface(gscreen, surface);
+#endif  /* ENABLE_X11 */
 
         g_object_unref(G_OBJECT(pix));
         cairo_destroy(cr);
@@ -735,8 +741,8 @@ workspace_backdrop_changed_cb(XfceWorkspace *workspace,
 }
 
 static void
-workspace_changed_cb(WnckScreen *wnck_screen,
-                     WnckWorkspace *previously_active_space,
+workspace_changed_cb(XfwScreen *xfw_screen,
+                     XfwWorkspace *previously_active_space,
                      gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
@@ -762,7 +768,7 @@ workspace_changed_cb(WnckScreen *wnck_screen,
     for(i = 0; i < xfce_desktop_get_n_monitors(desktop); i++) {
         backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[new_workspace], i);
         /* update it */
-        backdrop_changed_cb(backdrop, user_data);
+        backdrop_changed_cb(backdrop, desktop);
 
         /* When we're spanning screens we only care about the first monitor */
         if(xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[new_workspace]))
@@ -771,12 +777,13 @@ workspace_changed_cb(WnckScreen *wnck_screen,
 }
 
 static void
-workspace_created_cb(WnckScreen *wnck_screen,
-                     WnckWorkspace *new_workspace,
+workspace_created_cb(XfwScreen *screen,
+                     XfwWorkspace *new_workspace,
                      gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     gint nlast_workspace;
+
     TRACE("entering");
 
     nlast_workspace = desktop->priv->nworkspaces;
@@ -803,9 +810,9 @@ workspace_created_cb(WnckScreen *wnck_screen,
 }
 
 static void
-workspace_destroyed_cb(WnckScreen *wnck_screen,
-                     WnckWorkspace *old_workspace,
-                     gpointer user_data)
+workspace_destroyed_cb(XfwScreen *screen,
+                       XfwWorkspace *old_workspace,
+                       gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     gint nlast_workspace;
@@ -836,7 +843,47 @@ workspace_destroyed_cb(WnckScreen *wnck_screen,
 }
 
 static void
-screen_set_selection(XfceDesktop *desktop)
+workspace_group_created_cb(XfwScreen *screen,
+                           XfwWorkspaceGroup *group,
+                           gpointer user_data)
+{
+    XfceDesktop *desktop = XFCE_DESKTOP(user_data);
+
+    TRACE("entering");
+
+    g_signal_connect(group, "active-workspace-changed",
+                     G_CALLBACK(workspace_changed_cb), desktop);
+    g_signal_connect(group, "workspace-created",
+                     G_CALLBACK(workspace_created_cb), desktop);
+    g_signal_connect(group, "workspace-destroyed",
+                     G_CALLBACK(workspace_destroyed_cb), desktop);
+}
+
+static void
+workspace_group_destroyed_cb(XfwScreen *screen,
+                             XfwWorkspaceGroup *group,
+                             gpointer user_data)
+{
+    XfceDesktop *desktop = XFCE_DESKTOP(user_data);
+
+    TRACE("entering");
+
+    g_signal_handlers_disconnect_by_func(group, G_CALLBACK(workspace_changed_cb), desktop);
+    g_signal_handlers_disconnect_by_func(group, G_CALLBACK(workspace_created_cb), desktop);
+    g_signal_handlers_disconnect_by_func(group, G_CALLBACK(workspace_destroyed_cb), desktop);
+
+    for (GList *l = xfw_workspace_group_list_workspaces(group);
+         l != NULL;
+         l = l->next)
+    {
+        workspace_destroyed_cb(desktop->priv->xfw_screen, XFW_WORKSPACE(l->data), desktop);
+    }
+}
+
+
+#ifdef ENABLE_X11
+static void
+screen_set_x11_selection(XfceDesktop *desktop)
 {
     Window xwin;
     gint xscreen;
@@ -893,6 +940,7 @@ screen_set_selection(XfceDesktop *desktop)
         exit(1);
     }
 }
+#endif  /* ENABLE_X11 */
 
 
 
@@ -921,26 +969,6 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     widget_class->popup_menu = xfce_desktop_popup_menu;
     widget_class->style_updated = xfce_desktop_style_updated;
 
-    signals[SIG_POPULATE_ROOT_MENU] = g_signal_new("populate-root-menu",
-                                                   XFCE_TYPE_DESKTOP,
-                                                   G_SIGNAL_RUN_LAST,
-                                                   G_STRUCT_OFFSET(XfceDesktopClass,
-                                                                   populate_root_menu),
-                                                   NULL, NULL,
-                                                   g_cclosure_marshal_VOID__OBJECT,
-                                                   G_TYPE_NONE, 1,
-                                                   GTK_TYPE_MENU_SHELL);
-
-    signals[SIG_POPULATE_SECONDARY_ROOT_MENU] = g_signal_new("populate-secondary-root-menu",
-                                                             XFCE_TYPE_DESKTOP,
-                                                             G_SIGNAL_RUN_LAST,
-                                                             G_STRUCT_OFFSET(XfceDesktopClass,
-                                                                             populate_secondary_root_menu),
-                                                             NULL, NULL,
-                                                             g_cclosure_marshal_VOID__OBJECT,
-                                                             G_TYPE_NONE, 1,
-                                                             GTK_TYPE_MENU_SHELL);
-
 #define XFDESKTOP_PARAM_FLAGS  (G_PARAM_READWRITE \
                                 | G_PARAM_CONSTRUCT \
                                 | G_PARAM_STATIC_NAME \
@@ -970,7 +998,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     g_object_class_install_property(gobject_class, PROP_ICON_ON_PRIMARY,
                                     g_param_spec_boolean("primary",
                                                          "primary",
-                                                         "primary",
+                                                         "show icons on primary desktop",
                                                          FALSE,
                                                          XFDESKTOP_PARAM_FLAGS));
 
@@ -1033,6 +1061,10 @@ xfce_desktop_finalize(GObject *object)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(object);
 
+    if (desktop->priv->active_root_menu != NULL) {
+        gtk_menu_shell_deactivate(GTK_MENU_SHELL(desktop->priv->active_root_menu));
+    }
+
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
 
@@ -1156,9 +1188,8 @@ xfce_desktop_realize(GtkWidget *widget)
     XfceDesktop *desktop = XFCE_DESKTOP(widget);
     GdkAtom atom;
     gint sw, sh;
-    Window xid;
-    GdkWindow *groot;
-    WnckScreen *wnck_screen;
+    XfwScreen *xfw_screen;
+    XfwWorkspaceManager *workspace_manager;
 
     TRACE("entering");
 
@@ -1184,23 +1215,29 @@ xfce_desktop_realize(GtkWidget *widget)
             gdk_atom_intern("ATOM", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&atom, 1);
 
-    xid = GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(desktop)));
-    groot = gdk_screen_get_root_window(desktop->priv->gscreen);
+#ifdef ENABLE_X11
+    if (xfw_windowing_get() == XFW_WINDOWING_X11) {
+        Window xid = GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(desktop)));
+        GdkWindow *groot = gdk_screen_get_root_window(desktop->priv->gscreen);
 
-    gdk_property_change(groot,
-            gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE),
-            gdk_atom_intern("WINDOW", FALSE), 32,
-            GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
+        gdk_property_change(groot,
+                gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE),
+                gdk_atom_intern("WINDOW", FALSE), 32,
+                GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
 
-    gdk_property_change(groot,
-            gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
-            gdk_atom_intern("WINDOW", FALSE), 32,
-            GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
+        gdk_property_change(groot,
+                gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
+                gdk_atom_intern("WINDOW", FALSE), 32,
+                GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
 
-    screen_set_selection(desktop);
+        screen_set_x11_selection(desktop);
+    }
+#endif  /* ENABLE_X11 */
 
-    wnck_screen = wnck_screen_get(gdk_x11_screen_get_screen_number(desktop->priv->gscreen));
-    desktop->priv->wnck_screen = wnck_screen;
+    xfw_screen = xfw_screen_get_default();
+    desktop->priv->xfw_screen = xfw_screen;
+    workspace_manager = xfw_screen_get_workspace_manager(xfw_screen);
+    desktop->priv->workspace_manager = workspace_manager;
 
     /* Watch for single workspace setting changes */
     xfconf_g_property_bind(desktop->priv->channel,
@@ -1211,12 +1248,23 @@ xfce_desktop_realize(GtkWidget *widget)
                            G_OBJECT(desktop), "single-workspace-number");
 
     /* watch for workspace changes */
-    g_signal_connect(desktop->priv->wnck_screen, "active-workspace-changed",
-                     G_CALLBACK(workspace_changed_cb), desktop);
-    g_signal_connect(desktop->priv->wnck_screen, "workspace-created",
-                     G_CALLBACK(workspace_created_cb), desktop);
-    g_signal_connect(desktop->priv->wnck_screen, "workspace-destroyed",
-                     G_CALLBACK(workspace_destroyed_cb), desktop);
+    for (GList *gl = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+         gl != NULL;
+         gl = gl->next)
+    {
+        XfwWorkspaceGroup *group = XFW_WORKSPACE_GROUP(gl->data);
+        workspace_group_created_cb(xfw_screen, group, desktop);
+        for (GList *wl = xfw_workspace_group_list_workspaces(group);
+             wl != NULL;
+             wl = wl->next)
+        {
+            workspace_created_cb(xfw_screen, XFW_WORKSPACE(wl->data), desktop);
+        }
+    }
+    g_signal_connect(workspace_manager, "workspace-group-created",
+                     G_CALLBACK(workspace_group_created_cb), desktop);
+    g_signal_connect(workspace_manager, "workspace-group-destroyed",
+                     G_CALLBACK(workspace_group_destroyed_cb), desktop);
 
     /* watch for screen changes */
     g_signal_connect(G_OBJECT(desktop->priv->gscreen), "size-changed",
@@ -1230,6 +1278,8 @@ xfce_desktop_realize(GtkWidget *widget)
     xfce_desktop_setup_icon_view(desktop);
 #endif
 
+    xfce_desktop_refresh(desktop, FALSE, TRUE);
+
     TRACE("exiting");
 }
 
@@ -1256,7 +1306,7 @@ xfce_desktop_unrealize(GtkWidget *widget)
     gtk_widget_set_mapped(widget, FALSE);
 
     gtk_container_forall(GTK_CONTAINER(widget),
-                         (GtkCallback)gtk_widget_unrealize,
+                         xfdesktop_widget_unrealize,
                          NULL);
 
     g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
@@ -1265,7 +1315,7 @@ xfce_desktop_unrealize(GtkWidget *widget)
             G_CALLBACK(screen_composited_changed_cb), desktop);
 
     display = gdk_screen_get_display(desktop->priv->gscreen);
-    gdk_x11_display_error_trap_push(display);
+    xfw_windowing_error_trap_push(display);
 
     groot = gdk_screen_get_root_window(desktop->priv->gscreen);
     gdk_property_delete(groot, gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE));
@@ -1289,7 +1339,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     }
 
     gdk_display_flush(display);
-    gdk_x11_display_error_trap_pop_ignored(display);
+    xfw_windowing_error_trap_pop_ignored(display);
 
     if(desktop->priv->bg_surface) {
         cairo_surface_destroy(desktop->priv->bg_surface);
@@ -1519,15 +1569,24 @@ xfce_desktop_get_single_workspace_mode(XfceDesktop *desktop)
 static gint
 xfce_desktop_get_current_workspace(XfceDesktop *desktop)
 {
-    WnckWorkspace *wnck_workspace;
+    XfwWorkspace *workspace = NULL;
     gint workspace_num, current_workspace;
 
     g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), -1);
 
-    wnck_workspace = wnck_screen_get_active_workspace(desktop->priv->wnck_screen);
-
-    if(wnck_workspace != NULL) {
-        workspace_num = wnck_workspace_get_number(wnck_workspace);
+    // TODO: handle multiple workspace groups somehow
+    for (GList *l = xfw_workspace_manager_list_workspace_groups(desktop->priv->workspace_manager);
+         l != NULL;
+         l = l->next)
+    {
+        workspace = xfw_workspace_group_get_active_workspace(XFW_WORKSPACE_GROUP(l->data));
+        if (workspace != NULL) {
+            break;
+        }
+    }
+    if (workspace != NULL) {
+        gint n_ws;
+        xfdesktop_workspace_get_number_and_total(desktop->priv->workspace_manager, workspace, &workspace_num, &n_ws);
     } else {
         workspace_num = desktop->priv->nworkspaces;
     }
@@ -1694,6 +1753,7 @@ xfce_desktop_set_primary(XfceDesktop *desktop,
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
+#ifdef ENABLE_DESKTOP_ICONS
     if(primary == desktop->priv->primary)
         return;
 
@@ -1703,6 +1763,7 @@ xfce_desktop_set_primary(XfceDesktop *desktop,
         xfdesktop_icon_view_set_primary(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
                                         primary);
     }
+#endif
 }
 
 void
@@ -1837,18 +1898,35 @@ xfce_desktop_menu_destroy_idled(gpointer data)
     return FALSE;
 }
 
+static void
+xfce_desktop_menu_deactivated(GtkWidget *menu,
+                              XfceDesktop *desktop)
+{
+    if (desktop->priv->active_root_menu == menu) {
+        desktop->priv->active_root_menu = NULL;
+    }
+    g_idle_add(xfce_desktop_menu_destroy_idled, menu);
+}
+
 static void
 xfce_desktop_do_menu_popup(XfceDesktop *desktop,
                            guint button,
                            guint activate_time,
-                           guint populate_signal)
+                           gboolean populate_from_icon_view,
+                           PopulateMenuFunc populate_func)
 {
     GdkScreen *screen;
     GtkWidget *menu;
+    GtkWidget *actual_menu = NULL;
     GList *menu_children;
 
     DBG("entering");
 
+    if (desktop->priv->active_root_menu != NULL) {
+        gtk_menu_shell_deactivate(GTK_MENU_SHELL(desktop->priv->active_root_menu));
+        desktop->priv->active_root_menu = NULL;
+    }
+
     if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
         screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
     else
@@ -1857,29 +1935,42 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
     menu = gtk_menu_new();
     gtk_menu_set_screen(GTK_MENU(menu), screen);
     gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
-    g_signal_connect_swapped(G_OBJECT(menu), "deactivate",
-                             G_CALLBACK(g_idle_add),
-                             (gpointer)xfce_desktop_menu_destroy_idled);
 
-    g_signal_emit(G_OBJECT(desktop), populate_signal, 0, menu);
+#ifdef ENABLE_DESKTOP_ICONS
+    if (populate_from_icon_view && desktop->priv->icon_view != NULL) {
+        XfdesktopIconViewManager *manager = xfdesktop_icon_view_get_manager(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view));
+        xfdesktop_icon_view_manager_populate_context_menu(manager, GTK_MENU_SHELL(menu));
+    }
+#endif
 
-    /* if nobody populated the menu, don't do anything */
-    menu_children = gtk_container_get_children(GTK_CONTAINER(menu));
-    if(!menu_children) {
+    actual_menu = GTK_WIDGET((*populate_func)(GTK_MENU_SHELL(menu), gtk_widget_get_scale_factor(GTK_WIDGET(desktop))));
+    if (actual_menu != menu) {
+        g_object_ref_sink(menu);
         gtk_widget_destroy(menu);
-        return;
+        menu = NULL;
     }
 
-    g_list_free(menu_children);
+    /* if the toplevel is the garcon menu, it loads items asynchronously; calling _show() forces
+     * loading to complete.  otherwise, the _get_children() call would return NULL */
+    gtk_widget_show(actual_menu);
 
-    gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(desktop), NULL);
+    /* if nobody populated the menu, don't do anything */
+    menu_children = gtk_container_get_children(GTK_CONTAINER(actual_menu));
+    if (menu_children != NULL) {
+        g_list_free(menu_children);
 
-    /* Per gtk_menu_popup's documentation "for conflict-resolve initiation of
-     * concurrent requests for mouse/keyboard grab requests." */
-    if(activate_time == 0)
-        activate_time = gtk_get_current_event_time();
+        gtk_menu_attach_to_widget(GTK_MENU(actual_menu), GTK_WIDGET(desktop), NULL);
+        g_signal_connect(actual_menu, "deactivate",
+                         G_CALLBACK(xfce_desktop_menu_deactivated), desktop);
 
-    xfce_gtk_menu_popup_until_mapped(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
+        /* Per gtk_menu_popup's documentation "for conflict-resolve initiation of
+         * concurrent requests for mouse/keyboard grab requests." */
+        if(activate_time == 0)
+            activate_time = gtk_get_current_event_time();
+
+        desktop->priv->active_root_menu = actual_menu;
+        xfce_gtk_menu_popup_until_mapped(GTK_MENU(actual_menu), NULL, NULL, NULL, NULL, button, activate_time);
+    }
 }
 
 
@@ -1890,8 +1981,7 @@ xfce_desktop_popup_root_menu(XfceDesktop *desktop,
 {
     DBG("entering");
 
-    xfce_desktop_do_menu_popup(desktop, button, activate_time,
-                               signals[SIG_POPULATE_ROOT_MENU]);
+    xfce_desktop_do_menu_popup(desktop, button, activate_time, TRUE, menu_populate);
 
 }
 
@@ -1902,8 +1992,7 @@ xfce_desktop_popup_secondary_root_menu(XfceDesktop *desktop,
 {
     DBG("entering");
 
-    xfce_desktop_do_menu_popup(desktop, button, activate_time,
-                               signals[SIG_POPULATE_SECONDARY_ROOT_MENU]);
+    xfce_desktop_do_menu_popup(desktop, button, activate_time, FALSE, windowlist_populate);
 }
 
 void
