@@ -1,7 +1,7 @@
 /*
  *  xfworkspace - xfce4's desktop manager
  *
- *  Copyright (c) 2004-2007 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2007 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -57,9 +57,6 @@
 #include <time.h>
 #endif
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
 #include <glib.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
@@ -73,7 +70,7 @@
 #include "xfce-workspace.h"
 #include "xfce-desktop-enum-types.h"
 
-struct _XfceWorkspacePriv
+struct _XfceWorkspacePrivate
 {
     GdkScreen *gscreen;
 
@@ -116,7 +113,7 @@ static void xfce_workspace_disconnect_backdrop_settings(XfceWorkspace *workspace
 
 static void xfce_workspace_remove_backdrops(XfceWorkspace *workspace);
 
-G_DEFINE_TYPE(XfceWorkspace, xfce_workspace, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(XfceWorkspace, xfce_workspace, G_TYPE_OBJECT)
 
 
 /**
@@ -144,19 +141,13 @@ xfce_workspace_set_xfconf_property_string(XfceWorkspace *workspace,
 {
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
     GdkDisplay *display;
-#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
-    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    display = gdk_display_get_default();
     monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
-#else
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
-#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -183,21 +174,15 @@ xfce_workspace_set_xfconf_property_value(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gchar *monitor_name = NULL;
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
     GdkDisplay *display;
-#endif
 #ifdef G_ENABLE_DEBUG
     gchar *contents = NULL;
 #endif
 
     TRACE("entering");
 
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
-    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    display = gdk_display_get_default();
     monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
-#else
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
-#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -319,11 +304,7 @@ xfce_workspace_monitors_changed(XfceWorkspace *workspace,
         /* When spanning screens we only need one backdrop */
         n_monitors = 1;
     } else {
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
         n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
-#else
-        n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
-#endif
     }
 
     /* Remove all backdrops so that the correct monitor is added/removed and
@@ -366,8 +347,6 @@ xfce_workspace_class_init(XfceWorkspaceClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
 
-    g_type_class_add_private(klass, sizeof(XfceWorkspacePriv));
-
     gobject_class->finalize = xfce_workspace_finalize;
     gobject_class->set_property = xfce_workspace_set_property;
     gobject_class->get_property = xfce_workspace_get_property;
@@ -386,8 +365,7 @@ xfce_workspace_class_init(XfceWorkspaceClass *klass)
 static void
 xfce_workspace_init(XfceWorkspace *workspace)
 {
-    workspace->priv = G_TYPE_INSTANCE_GET_PRIVATE(workspace, XFCE_TYPE_WORKSPACE,
-                                                XfceWorkspacePriv);
+    workspace->priv = xfce_workspace_get_instance_private(workspace);
 }
 
 static void
@@ -628,19 +606,13 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gint pp_len;
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
     GdkDisplay *display;
-#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
-    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    display = gdk_display_get_default();
     monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor)));
-#else
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor);
-#endif
 
     if(monitor_name == NULL) {
         g_snprintf(buf, sizeof(buf), "%smonitor%d/workspace%d/",
@@ -705,11 +677,6 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
                            G_OBJECT(backdrop), "backdrop-cycle-random-order");
 
-    buf[pp_len] = 0;
-    g_strlcat(buf, "backdrop-do-animations", sizeof(buf));
-    xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
-                           G_OBJECT(backdrop), "backdrop-do-animations");
-
     buf[pp_len] = 0;
     g_strlcat(buf, "last-image", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
@@ -736,17 +703,10 @@ static void
 xfce_workspace_remove_backdrops(XfceWorkspace *workspace)
 {
     guint i;
-    guint n_monitors;
 
     g_return_if_fail(XFCE_IS_WORKSPACE(workspace));
 
-#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
-    n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
-#else
-    n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
-#endif
-
-    for(i = 0; i < n_monitors && i < workspace->priv->nbackdrops; ++i) {
+    for(i = 0; i < workspace->priv->nbackdrops; ++i) {
         xfce_workspace_disconnect_backdrop_settings(workspace,
                                                     workspace->priv->backdrops[i],
                                                     i);
@@ -787,7 +747,7 @@ xfce_workspace_new(GdkScreen *gscreen,
 
     workspace->priv->gscreen = gscreen;
     workspace->priv->workspace_num = number;
-    workspace->priv->channel = g_object_ref(G_OBJECT(channel));
+    workspace->priv->channel = XFCONF_CHANNEL(g_object_ref(G_OBJECT(channel)));
     workspace->priv->property_prefix = g_strdup(property_prefix);
 
     return workspace;
