@@ -1,7 +1,8 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006      Brian Tarricone, <brian@tarricone.org>
+ *  Copyright (c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,17 +15,16 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_FILE_ICON_H__
 #define __XFDESKTOP_FILE_ICON_H__
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 
-#include <thunar-vfs/thunar-vfs.h>
-
 #include "xfdesktop-icon.h"
 
 G_BEGIN_DECLS
@@ -41,7 +41,7 @@ typedef struct _XfdesktopFileIconPrivate XfdesktopFileIconPrivate;
 struct _XfdesktopFileIcon
 {
     XfdesktopIcon parent;
-    
+
     /*< private >*/
     XfdesktopFileIconPrivate *priv;
 };
@@ -49,36 +49,34 @@ struct _XfdesktopFileIcon
 struct _XfdesktopFileIconClass
 {
     XfdesktopIconClass parent;
-    
+
     /*< virtual functions >*/
-    G_CONST_RETURN ThunarVfsInfo *(*peek_info)(XfdesktopFileIcon *icon);
-    void (*update_info)(XfdesktopFileIcon *icon, ThunarVfsInfo *info);
-    
+    GFileInfo *(*peek_file_info)(XfdesktopFileIcon *icon);
+    GFileInfo *(*peek_filesystem_info)(XfdesktopFileIcon *icon);
+    GFile *(*peek_file)(XfdesktopFileIcon *icon);
+    void (*update_file_info)(XfdesktopFileIcon *icon, GFileInfo *info);
+
     gboolean (*can_rename_file)(XfdesktopFileIcon *icon);
-    gboolean (*rename_file)(XfdesktopFileIcon *icon, const gchar *new_name);
-    
     gboolean (*can_delete_file)(XfdesktopFileIcon *icon);
-    gboolean (*delete_file)(XfdesktopFileIcon *icon);
 };
 
 GType xfdesktop_file_icon_get_type(void) G_GNUC_CONST;
 
-G_CONST_RETURN ThunarVfsInfo *xfdesktop_file_icon_peek_info(XfdesktopFileIcon *icon);
-void xfdesktop_file_icon_update_info(XfdesktopFileIcon *icon,
-                                     ThunarVfsInfo *info);
+GFileInfo *xfdesktop_file_icon_peek_file_info(XfdesktopFileIcon *icon);
+GFileInfo *xfdesktop_file_icon_peek_filesystem_info(XfdesktopFileIcon *icon);
+GFile *xfdesktop_file_icon_peek_file(XfdesktopFileIcon *icon);
+void xfdesktop_file_icon_update_file_info(XfdesktopFileIcon *icon,
+                                          GFileInfo *info);
 
 gboolean xfdesktop_file_icon_can_rename_file(XfdesktopFileIcon *icon);
-gboolean xfdesktop_file_icon_rename_file(XfdesktopFileIcon *icon,
-                                         const gchar *new_name);
 
 gboolean xfdesktop_file_icon_can_delete_file(XfdesktopFileIcon *icon);
-gboolean xfdesktop_file_icon_delete_file(XfdesktopFileIcon *icon);
 
-void xfdesktop_file_icon_add_active_job(XfdesktopFileIcon *icon,
-                                        ThunarVfsJob *job);
-gboolean xfdesktop_file_icon_remove_active_job(XfdesktopFileIcon *icon,
-                                               ThunarVfsJob *job);
+GIcon *xfdesktop_file_icon_add_emblems(XfdesktopFileIcon *icon);
 
+void xfdesktop_file_icon_invalidate_icon(XfdesktopFileIcon *icon);
+
+gboolean xfdesktop_file_icon_has_gicon(XfdesktopFileIcon *icon);
 
 G_END_DECLS
 
