@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_ICON_VIEW_MANAGER_H__
@@ -42,23 +42,23 @@ struct _XfdesktopIconView;
 struct _XfdesktopIconViewManagerIface
 {
     GTypeInterface g_iface;
-    
+
     /*< virtual functions >*/
     gboolean (*manager_init)(XfdesktopIconViewManager *manager,
                              struct _XfdesktopIconView *icon_view);
     void (*manager_fini)(XfdesktopIconViewManager *manager);
-    
+
     gboolean (*drag_drop)(XfdesktopIconViewManager *manager,
                           XfdesktopIcon *drop_icon,
                           GdkDragContext *context,
-                          guint16 row,
-                          guint16 col,
+                          gint16 row,
+                          gint16 col,
                           guint time_);
     void (*drag_data_received)(XfdesktopIconViewManager *manager,
                                XfdesktopIcon *drop_icon,
                                GdkDragContext *context,
-                               guint16 row,
-                               guint16 col,
+                               gint16 row,
+                               gint16 col,
                                GtkSelectionData *data,
                                guint info,
                                guint time_);
@@ -74,6 +74,8 @@ struct _XfdesktopIconViewManagerIface
                                          GdkDragContext *context,
                                          GtkSelectionData *data,
                                          guint info);
+    void (*populate_context_menu)(XfdesktopIconViewManager *manager,
+                                  GtkMenuShell *menu);
 };
 
 GType xfdesktop_icon_view_manager_get_type(void) G_GNUC_CONST;
@@ -87,14 +89,14 @@ void xfdesktop_icon_view_manager_fini(XfdesktopIconViewManager *manager);
 gboolean xfdesktop_icon_view_manager_drag_drop(XfdesktopIconViewManager *manager,
                                                XfdesktopIcon *drop_icon,
                                                GdkDragContext *context,
-                                               guint16 row,
-                                               guint16 col,
+                                               gint16 row,
+                                               gint16 col,
                                                guint time_);
 void xfdesktop_icon_view_manager_drag_data_received(XfdesktopIconViewManager *manager,
                                                     XfdesktopIcon *drop_icon,
                                                     GdkDragContext *context,
-                                                    guint16 row,
-                                                    guint16 col,
+                                                    gint16 row,
+                                                    gint16 col,
                                                     GtkSelectionData *data,
                                                     guint info,
                                                     guint time_);
@@ -111,7 +113,8 @@ GdkDragAction xfdesktop_icon_view_manager_propose_drop_action(XfdesktopIconViewM
                                                               GtkSelectionData *data,
                                                               guint info);
 
-
+void xfdesktop_icon_view_manager_populate_context_menu(XfdesktopIconViewManager *manager,
+                                                       GtkMenuShell *menu);
 
 G_END_DECLS
 
