@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ typedef void (*XfdesktopIconViewIconFiniFunc)(XfdesktopIconView *icon_view);
 struct _XfdesktopIconView
 {
     GtkWidget parent;
-    
+
     /*< private >*/
     XfdesktopIconViewPrivate *priv;
 };
@@ -50,7 +50,7 @@ struct _XfdesktopIconView
 struct _XfdesktopIconViewClass
 {
     GtkWidgetClass parent;
-    
+
     /*< signals >*/
     void (*icon_selection_changed)(XfdesktopIconView *icon_view);
     void (*icon_activated)(XfdesktopIconView *icon_view);
@@ -61,7 +61,7 @@ struct _XfdesktopIconViewClass
     void (*select_cursor_item)(XfdesktopIconView *icon_view);
     void (*toggle_cursor_item)(XfdesktopIconView *icon_view);
 
-    gboolean (*activate_cursor_item)(XfdesktopIconView *icon_view);
+    gboolean (*activate_selected_items)(XfdesktopIconView *icon_view);
 
     gboolean (*move_cursor)(XfdesktopIconView *icon_view,
                             GtkMovementStep step,
@@ -74,6 +74,8 @@ GType xfdesktop_icon_view_get_type(void) G_GNUC_CONST;
 
 GtkWidget *xfdesktop_icon_view_new(XfdesktopIconViewManager *manager);
 
+XfdesktopIconViewManager *xfdesktop_icon_view_get_manager(XfdesktopIconView *icon_view);
+
 void xfdesktop_icon_view_add_item(XfdesktopIconView *icon_view,
                                   XfdesktopIcon *icon);
 
@@ -114,9 +116,14 @@ void xfdesktop_icon_view_set_icon_size(XfdesktopIconView *icon_view,
                                        guint icon_size);
 guint xfdesktop_icon_view_get_icon_size(XfdesktopIconView *icon_view);
 
+void xfdesktop_icon_view_set_primary(XfdesktopIconView *icon_view,
+                                     gboolean primary);
+
 void xfdesktop_icon_view_set_font_size(XfdesktopIconView *icon_view,
                                        gdouble font_size_points);
 gdouble xfdesktop_icon_view_get_font_size(XfdesktopIconView *icon_view);
+void xfdesktop_icon_view_set_center_text (XfdesktopIconView *icon_view,
+                                          gboolean center_text);
 
 GtkWidget *xfdesktop_icon_view_get_window_widget(XfdesktopIconView *icon_view);
 
