@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006-2007 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006-2007 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ static guint __signals[SIG_N_SIGNALS] = { 0, };
 
 static void xfdesktop_icon_finalize(GObject *obj);
 
-G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 
 
 static void
@@ -68,8 +68,6 @@ xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
 
-    g_type_class_add_private(klass, sizeof(XfdesktopIconPrivate));
-
     gobject_class->finalize = xfdesktop_icon_finalize;
 
     __signals[SIG_PIXBUF_CHANGED] = g_signal_new("pixbuf-changed",
@@ -122,8 +120,7 @@ xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 static void
 xfdesktop_icon_init(XfdesktopIcon *icon)
 {
-    icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon, XFDESKTOP_TYPE_ICON,
-                                             XfdesktopIconPrivate);
+    icon->priv = xfdesktop_icon_get_instance_private(icon);
 }
 
 static void
@@ -285,7 +282,7 @@ xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
 /*< optional; required if get_allowed_drop_actions() can return nonzero >*/
 gboolean
 xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
-                            XfdesktopIcon *src_icon,
+                            GList *src_icons,
                             GdkDragAction action)
 {
     XfdesktopIconClass *klass;
@@ -294,7 +291,7 @@ xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
     klass = XFDESKTOP_ICON_GET_CLASS(icon);
     g_return_val_if_fail(klass->do_drop_dest, FALSE);
 
-    return klass->do_drop_dest(icon, src_icon, action);
+    return klass->do_drop_dest(icon, src_icons, action);
 }
 
 /*< optional >*/
@@ -461,3 +458,10 @@ xfdesktop_icon_activated(XfdesktopIcon *icon)
 
     return ret;
 }
+
+void
+xfdesktop_icon_activated_g_func(gpointer data,
+                                gpointer user_data)
+{
+    xfdesktop_icon_activated(XFDESKTOP_ICON (data));
+}
