@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ typedef struct _XfdesktopIconPrivate XfdesktopIconPrivate;
 struct _XfdesktopIcon
 {
     GObject parent;
-    
+
     /*< private >*/
     XfdesktopIconPrivate *priv;
 };
@@ -46,13 +46,13 @@ struct _XfdesktopIcon
 struct _XfdesktopIconClass
 {
     GObjectClass parent;
-    
+
     /*< signals >*/
     void (*pixbuf_changed)(XfdesktopIcon *icon);
     void (*label_changed)(XfdesktopIcon *icon);
-    
+
     void (*position_changed)(XfdesktopIcon *icon);
-    
+
     void (*selected)(XfdesktopIcon *icon);
     /* XfdektopIcon::activated has weird semantics: you should NEVER connect to
      * this signal normally: always use g_signal_connect_after(), as the default
@@ -61,21 +61,21 @@ struct _XfdesktopIconClass
      * with return values are (for some unknown reason) not allowed to be
      * G_SIGNAL_RUN_FIRST.  go figure. */
     gboolean (*activated)(XfdesktopIcon *icon);
-    
+
     /*< virtual functions >*/
     GdkPixbuf *(*peek_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
-    G_CONST_RETURN gchar *(*peek_label)(XfdesktopIcon *icon);
-    
+    const gchar *(*peek_label)(XfdesktopIcon *icon);
+
     GdkDragAction (*get_allowed_drag_actions)(XfdesktopIcon *icon);
-    
+
     GdkDragAction (*get_allowed_drop_actions)(XfdesktopIcon *icon, GdkDragAction *suggested_action);
-    gboolean (*do_drop_dest)(XfdesktopIcon *icon, XfdesktopIcon *src_icon, GdkDragAction action);
+    gboolean (*do_drop_dest)(XfdesktopIcon *icon, GList *src_icons, GdkDragAction action);
 
     GdkPixbuf *(*peek_tooltip_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
-    G_CONST_RETURN gchar *(*peek_tooltip)(XfdesktopIcon *icon);
+    const gchar *(*peek_tooltip)(XfdesktopIcon *icon);
 
     gchar *(*get_identifier)(XfdesktopIcon *icon);
-    
+
     void (*set_thumbnail_file)(XfdesktopIcon *icon, GFile *file);
     void (*delete_thumbnail_file)(XfdesktopIcon *icon);
 
@@ -90,11 +90,11 @@ GType xfdesktop_icon_get_type(void) G_GNUC_CONST;
 GdkPixbuf *xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
                                      gint width,
                                      gint height);
-G_CONST_RETURN gchar *xfdesktop_icon_peek_label(XfdesktopIcon *icon);
+const gchar *xfdesktop_icon_peek_label(XfdesktopIcon *icon);
 GdkPixbuf *xfdesktop_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
                                               gint width,
                                               gint height);
-G_CONST_RETURN gchar *xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon);
+const gchar *xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon);
 
 /* returns a unique identifier for the icon, free when done using it */
 gchar *xfdesktop_icon_get_identifier(XfdesktopIcon *icon);
@@ -103,15 +103,15 @@ void xfdesktop_icon_set_position(XfdesktopIcon *icon,
                                  gint16 row,
                                  gint16 col);
 gboolean xfdesktop_icon_get_position(XfdesktopIcon *icon,
-                                     guint16 *row,
-                                     guint16 *col);
+                                     gint16 *row,
+                                     gint16 *col);
 
 GdkDragAction xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
 
 GdkDragAction xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
                                                       GdkDragAction *suggested_action);
 gboolean xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
-                                     XfdesktopIcon *src_icon,
+                                     GList *src_icons,
                                      GdkDragAction action);
 
 gboolean xfdesktop_icon_populate_context_menu(XfdesktopIcon *icon,
@@ -134,6 +134,8 @@ void xfdesktop_icon_position_changed(XfdesktopIcon *icon);
 
 void xfdesktop_icon_selected(XfdesktopIcon *icon);
 gboolean xfdesktop_icon_activated(XfdesktopIcon *icon);
+void xfdesktop_icon_activated_g_func(gpointer data,
+                                     gpointer user_data);
 
 /*< private-ish; only for use by XfdesktopIconView >*/
 void xfdesktop_icon_set_extents(XfdesktopIcon *icon,
