@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright(c) 2006      Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright(c) 2006      Brian Tarricone, <brian@tarricone.org>
  *  Copyright(c) 2006      Benedikt Meurer, <benny@xfce.org>
  *  Copyright(c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
@@ -61,7 +61,7 @@ struct _XfdesktopSpecialFileIconPrivate
     GFileInfo *filesystem_info;
     GFile *file;
     GdkScreen *gscreen;
-    
+
     /* only needed for trash */
     guint trash_item_count;
 };
@@ -70,16 +70,16 @@ static void xfdesktop_special_file_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
                                                           gint width, gint height);
-static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
+static const gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
 static gchar *xfdesktop_special_file_icon_get_identifier(XfdesktopIcon *icon);
 static GdkPixbuf *xfdesktop_special_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
                                                                   gint width, gint height);
-static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
+static const gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
                                                                           GdkDragAction *suggested_action);
 static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
-                                                         XfdesktopIcon *src_icon,
+                                                         GList *src_icons,
                                                          GdkDragAction action);
 static gboolean xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
                                                                   GtkWidget *menu);
@@ -101,10 +101,11 @@ G_DEFINE_TYPE_EXTENDED(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
                                              xfdesktop_special_file_icon_tfi_init)
+                       G_ADD_PRIVATE(XfdesktopSpecialFileIcon)
                        )
 #else
-G_DEFINE_TYPE(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
-              XFDESKTOP_TYPE_FILE_ICON)
+G_DEFINE_TYPE_WITH_PRIVATE(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
+                           XFDESKTOP_TYPE_FILE_ICON)
 #endif
 
 
@@ -115,11 +116,9 @@ xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
     GObjectClass *gobject_class = (GObjectClass *)klass;
     XfdesktopIconClass *icon_class = (XfdesktopIconClass *)klass;
     XfdesktopFileIconClass *file_icon_class = (XfdesktopFileIconClass *)klass;
-    
-    g_type_class_add_private(klass, sizeof(XfdesktopSpecialFileIconPrivate));
-    
+
     gobject_class->finalize = xfdesktop_special_file_icon_finalize;
-    
+
     icon_class->peek_pixbuf = xfdesktop_special_file_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_special_file_icon_peek_label;
     icon_class->get_identifier = xfdesktop_special_file_icon_get_identifier;
@@ -129,20 +128,16 @@ xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
     icon_class->get_allowed_drop_actions = xfdesktop_special_file_icon_get_allowed_drop_actions;
     icon_class->do_drop_dest = xfdesktop_special_file_icon_do_drop_dest;
     icon_class->populate_context_menu = xfdesktop_special_file_icon_populate_context_menu;
-    
+
     file_icon_class->peek_file_info = xfdesktop_special_file_icon_peek_file_info;
     file_icon_class->peek_filesystem_info = xfdesktop_special_file_icon_peek_filesystem_info;
     file_icon_class->peek_file = xfdesktop_special_file_icon_peek_file;
-    file_icon_class->can_rename_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
-    file_icon_class->can_delete_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
 }
 
 static void
 xfdesktop_special_file_icon_init(XfdesktopSpecialFileIcon *icon)
 {
-    icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon,
-                                             XFDESKTOP_TYPE_SPECIAL_FILE_ICON,
-                                             XfdesktopSpecialFileIconPrivate);
+    icon->priv = xfdesktop_special_file_icon_get_instance_private(icon);
 }
 
 static void
@@ -150,7 +145,7 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
 {
     XfdesktopSpecialFileIcon *icon = XFDESKTOP_SPECIAL_FILE_ICON(obj);
     GtkIconTheme *itheme = gtk_icon_theme_get_for_screen(icon->priv->gscreen);
-    
+
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
                                          G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                                          icon);
@@ -161,7 +156,7 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
                                              icon);
         g_object_unref(icon->priv->monitor);
     }
-    
+
     g_object_unref(icon->priv->file);
 
     if(icon->priv->file_info)
@@ -169,7 +164,7 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
 
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
-    
+
     G_OBJECT_CLASS(xfdesktop_special_file_icon_parent_class)->finalize(obj);
 }
 
@@ -178,7 +173,7 @@ static gchar *
 xfdesktop_special_file_icon_tfi_get_uri_scheme(ThunarxFileInfo *file_info)
 {
     XfdesktopSpecialFileIcon *icon = XFDESKTOP_SPECIAL_FILE_ICON(file_info);
-    
+
     if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == icon->priv->type)
         return g_strdup("trash");
     else
@@ -285,16 +280,18 @@ xfdesktop_special_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
     return tooltip_pix;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     GFileInfo *info = special_file_icon->priv->file_info;
-    
+
     if(XFDESKTOP_SPECIAL_FILE_ICON_HOME == special_file_icon->priv->type)
         return _("Home");
     else if(XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM == special_file_icon->priv->type)
         return _("File System");
+    else if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == special_file_icon->priv->type)
+        return _("Trash");
     else
         return info ? g_file_info_get_display_name(info) : NULL;
 }
@@ -317,27 +314,27 @@ xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     GdkDragAction actions = 0;
-    
+
     switch(special_file_icon->priv->type) {
         case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
             /* move is just impossible, and copy seems a bit retarded.  link
              * is possible */
             actions = GDK_ACTION_LINK;
             break;
-        
+
         case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
             /* user shouldn't be able to move their own homedir.  copy might
              * be a little silly, but allow it anyway.  link is fine. */
             actions = GDK_ACTION_COPY | GDK_ACTION_LINK;
             break;
-            
+
         case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
             /* move is impossible, but we can copy and link the trash root
              * anywhere */
             actions = GDK_ACTION_COPY | GDK_ACTION_LINK;
             break;
     }
-    
+
     return actions;
 }
 
@@ -355,19 +352,19 @@ xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
             if(g_file_info_get_attribute_boolean(info,
                                                  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
             {
-                DBG("can move, copy and link");
-                actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+                XF_DEBUG("can move, copy, link and ask");
+                actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK;
                 if(suggested_action)
                     *suggested_action = GDK_ACTION_MOVE;
             }
         }
     } else {
-        DBG("can move");
+        XF_DEBUG("can move");
         actions = GDK_ACTION_MOVE; /* everything else is just silly */
         if(suggested_action)
             *suggested_action = GDK_ACTION_MOVE;
     }
-    
+
     if(suggested_action)
         *suggested_action = 0;
 
@@ -376,81 +373,61 @@ xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
 
 static gboolean
 xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
-                                         XfdesktopIcon *src_icon,
+                                         GList *src_icons,
                                          GdkDragAction action)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    XfdesktopFileIcon *src_file_icon = XFDESKTOP_FILE_ICON(src_icon);
-    GFileInfo *src_info;
-    GFile *src_file;
-    GFile *dest_file = NULL;
     gboolean result = FALSE;
-    
-    DBG("entering");
-    
-    g_return_val_if_fail(special_file_icon && src_file_icon, FALSE);
+
+    TRACE("entering");
+
+    g_return_val_if_fail(special_file_icon != NULL && src_icons != NULL, FALSE);
     g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon, NULL),
                          FALSE);
-    
-    src_file = xfdesktop_file_icon_peek_file(src_file_icon);
 
-    src_info = xfdesktop_file_icon_peek_file_info(src_file_icon);
-    if(!src_info)
-        return FALSE;
-    
     if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
-        GList files;
+        GList *files = NULL;
 
-        DBG("doing trash");
+        XF_DEBUG("doing trash");
 
-        /* fake a file list */
-        files.data = src_file;
-        files.prev = files.next = NULL;
-
-        /* let the trash service handle the trash operation */
-        xfdesktop_file_utils_trash_files(&files, special_file_icon->priv->gscreen, NULL);
-    } else {
-        gchar *name = g_file_get_basename(src_file);
-        if(!name)
-            return FALSE;
-    
-        switch(action) {
-            case GDK_ACTION_MOVE:
-                DBG("doing move");
-                dest_file = g_object_ref(special_file_icon->priv->file);
-                break;
-            case GDK_ACTION_COPY:
-                DBG("doing copy");
-                dest_file = g_file_get_child(special_file_icon->priv->file, name);
-                break;
-            case GDK_ACTION_LINK:
-                DBG("doing link");
-                dest_file = g_object_ref(special_file_icon->priv->file);
-                break;
-            default:
-                g_warning("Unsupported drag action: %d", action);
+        for (GList *l = src_icons; l != NULL; l = l->next) {
+            GFile *file = xfdesktop_file_icon_peek_file(XFDESKTOP_FILE_ICON(l->data));
+            if (file != NULL) {
+                files = g_list_prepend(files, file);
+            }
         }
+        files = g_list_reverse(files);
+
+        if (files != NULL) {
+            /* let the trash service handle the trash operation */
+            xfdesktop_file_utils_trash_files(files, special_file_icon->priv->gscreen, NULL);
+            result = TRUE;
+        }
+    } else {
+        GList *src_files = NULL;
+        GList *dest_files = NULL;
+
+        xfdesktop_file_utils_build_transfer_file_lists(action, src_icons, XFDESKTOP_FILE_ICON(icon), &src_files, &dest_files);
 
         /* let the file manager service move/copy/link the file */
-        if(dest_file) {
-            xfdesktop_file_utils_transfer_file(action, src_file, dest_file,
+        if (src_files != NULL && dest_files != NULL) {
+            xfdesktop_file_utils_transfer_files(action, src_files, dest_files,
                                                special_file_icon->priv->gscreen);
-
             result = TRUE;
         }
 
-        g_object_unref(dest_file);
-        g_free(name);
+        g_list_free_full(dest_files, g_object_unref);
+        g_list_free(src_files);
     }
 
     return result;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    
+
     if(!special_file_icon->priv->tooltip) {
         GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
 
@@ -464,7 +441,7 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
                 special_file_icon->priv->tooltip = g_strdup_printf(g_dngettext(GETTEXT_PACKAGE,
                                                                                _("Trash contains one item"),
                                                                                _("Trash contains %d items"),
-                                                                               special_file_icon->priv->trash_item_count), 
+                                                                               special_file_icon->priv->trash_item_count),
 
                                                                    special_file_icon->priv->trash_item_count);
             }
@@ -491,7 +468,7 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
                                                      G_FILE_ATTRIBUTE_TIME_MODIFIED);
             time_string = xfdesktop_file_utils_format_time_for_display(mtime);
 
-            special_file_icon->priv->tooltip = 
+            special_file_icon->priv->tooltip =
                 g_strdup_printf(_("%s\nSize: %s\nLast modified: %s"),
                                 description, size_string, time_string);
 
@@ -499,7 +476,7 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
             g_free(time_string);
         }
     }
-    
+
     return special_file_icon->priv->tooltip;
 }
 
@@ -512,7 +489,7 @@ xfdesktop_special_file_icon_trash_open(GtkWidget *w,
 
     icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(file_icon));
     toplevel = gtk_widget_get_toplevel(icon_view);
-    
+
     xfdesktop_file_utils_open_folder(file_icon->priv->file,
                                      file_icon->priv->gscreen,
                                      GTK_WINDOW(toplevel));
@@ -538,19 +515,17 @@ xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     GtkWidget *mi, *img;
-    
+
     if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH != special_file_icon->priv->type)
         return FALSE;
-    
-    img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
-    gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Open"));
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+
+    img = gtk_image_new_from_icon_name("document-open", GTK_ICON_SIZE_MENU);
+    mi = xfdesktop_menu_create_menu_item_with_mnemonic(_("_Open"), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     g_signal_connect(G_OBJECT(mi), "activate",
                      G_CALLBACK(xfdesktop_special_file_icon_trash_open), icon);
-    
+
     mi = gtk_separator_menu_item_new();
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
@@ -560,10 +535,8 @@ xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
     } else {
         img = gtk_image_new_from_icon_name("user-trash-full", GTK_ICON_SIZE_MENU);
     }
-    
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Empty Trash"));
-    if(img)
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+
+    mi = xfdesktop_menu_create_menu_item_with_mnemonic(_("_Empty Trash"), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     if(special_file_icon->priv->trash_item_count > 0) {
@@ -572,7 +545,7 @@ xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
                          icon);
     } else
         gtk_widget_set_sensitive(mi, FALSE);
-    
+
     return TRUE;
 }
 
@@ -608,6 +581,12 @@ xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
     g_return_if_fail(G_IS_FILE(file));
     g_return_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(special_file_icon));
 
+    /* We don't care about change events only created/deleted */
+    if(event == G_FILE_MONITOR_EVENT_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
+        return;
+
     /* release the old file information */
     if(special_file_icon->priv->file_info) {
         g_object_unref(special_file_icon->priv->file_info);
@@ -621,7 +600,7 @@ xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
     }
 
     /* reload the file information */
-    special_file_icon->priv->file_info = g_file_query_info(special_file_icon->priv->file, 
+    special_file_icon->priv->file_info = g_file_query_info(special_file_icon->priv->file,
                                                            XFDESKTOP_FILE_INFO_NAMESPACE,
                                                            G_FILE_QUERY_INFO_NONE,
                                                            NULL, NULL);
@@ -695,30 +674,30 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
 {
     XfdesktopSpecialFileIcon *special_file_icon;
     GFile *file = NULL;
-    
+
     switch(type) {
         case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
             file = g_file_new_for_uri("file:///");
             break;
-        
+
         case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
             file = g_file_new_for_path(xfce_get_homedir());
             break;
-        
+
         case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
             file = g_file_new_for_uri("trash:///");
             break;
-        
+
         default:
             g_return_val_if_reached(NULL);
     }
-    
+
     special_file_icon = g_object_new(XFDESKTOP_TYPE_SPECIAL_FILE_ICON, NULL);
     special_file_icon->priv->type = type;
     special_file_icon->priv->gscreen = screen;
     special_file_icon->priv->file = file;
 
-    special_file_icon->priv->file_info = g_file_query_info(file, 
+    special_file_icon->priv->file_info = g_file_query_info(file,
                                                            XFDESKTOP_FILE_INFO_NAMESPACE,
                                                            G_FILE_QUERY_INFO_NONE,
                                                            NULL, NULL);
@@ -750,7 +729,7 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
                          G_CALLBACK(xfdesktop_special_file_icon_changed),
                          special_file_icon);
     }
-    
+
     return special_file_icon;
 }
 
