@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,16 +14,14 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_VOLUME_ICON_H__
 #define __XFDESKTOP_VOLUME_ICON_H__
 
-#include <glib-object.h>
-
-#include <thunar-vfs/thunar-vfs.h>
+#include <gio/gio.h>
 
 #include "xfdesktop-file-icon.h"
 
@@ -40,7 +38,7 @@ typedef struct _XfdesktopVolumeIconPrivate  XfdesktopVolumeIconPrivate;
 struct _XfdesktopVolumeIcon
 {
     XfdesktopFileIcon parent;
-    
+
     /*< private >*/
     XfdesktopVolumeIconPrivate *priv;
 };
@@ -52,10 +50,10 @@ struct _XfdesktopVolumeIconClass
 
 GType xfdesktop_volume_icon_get_type(void) G_GNUC_CONST;
 
-XfdesktopVolumeIcon *xfdesktop_volume_icon_new(ThunarVfsVolume *volume,
+XfdesktopVolumeIcon *xfdesktop_volume_icon_new(GVolume *volume,
                                                GdkScreen *screen);
 
-G_CONST_RETURN ThunarVfsVolume *xfdesktop_volume_icon_peek_volume(XfdesktopVolumeIcon *icon);
+GVolume *xfdesktop_volume_icon_peek_volume(XfdesktopVolumeIcon *icon);
 
 
 G_END_DECLS
