@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,16 +26,18 @@
 #include <string.h>
 #endif
 
-#include <libwnck/libwnck.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4windowing/libxfce4windowing.h>
+#include <libxfce4windowingui/libxfce4windowingui.h>
 
+#include "xfdesktop-common.h"
 #include "xfdesktop-window-icon.h"
 
 struct _XfdesktopWindowIconPrivate
 {
     gint workspace;
     gchar *label;
-    WnckWindow *window;
+    XfwWindow *window;
 };
 
 static void xfdesktop_window_icon_finalize(GObject *obj);
@@ -49,13 +51,13 @@ static gboolean xfdesktop_window_icon_activated(XfdesktopIcon *icon);
 static gboolean xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
                                                             GtkWidget *menu);
 
-static void xfdesktop_window_name_changed_cb(WnckWindow *window,
+static void xfdesktop_window_name_changed_cb(XfwWindow *window,
                                              gpointer user_data);
-static void xfdesktop_window_icon_changed_cb(WnckWindow *window,
+static void xfdesktop_window_icon_changed_cb(XfwWindow *window,
                                              gpointer user_data);
 
 
-G_DEFINE_TYPE(XfdesktopWindowIcon, xfdesktop_window_icon, XFDESKTOP_TYPE_ICON)
+G_DEFINE_TYPE_WITH_PRIVATE(XfdesktopWindowIcon, xfdesktop_window_icon, XFDESKTOP_TYPE_ICON)
 
 
 static void
@@ -64,8 +66,6 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
     GObjectClass *gobject_class = (GObjectClass *)klass;
     XfdesktopIconClass *icon_class = (XfdesktopIconClass *)klass;
 
-    g_type_class_add_private(klass, sizeof(XfdesktopWindowIconPrivate));
-
     gobject_class->finalize = xfdesktop_window_icon_finalize;
 
     icon_class->peek_pixbuf = xfdesktop_window_icon_peek_pixbuf;
@@ -78,8 +78,7 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
 static void
 xfdesktop_window_icon_init(XfdesktopWindowIcon *icon)
 {
-    icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon, XFDESKTOP_TYPE_WINDOW_ICON,
-                                             XfdesktopWindowIconPrivate);
+    icon->priv = xfdesktop_window_icon_get_instance_private(icon);
 }
 
 static void
@@ -116,7 +115,7 @@ xfdesktop_window_icon_finalize(GObject *obj)
 
 
 static void
-xfdesktop_window_name_changed_cb(WnckWindow *window,
+xfdesktop_window_name_changed_cb(XfwWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
@@ -128,7 +127,7 @@ xfdesktop_window_name_changed_cb(WnckWindow *window,
 }
 
 static void
-xfdesktop_window_icon_changed_cb(WnckWindow *window,
+xfdesktop_window_icon_changed_cb(XfwWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
@@ -143,8 +142,9 @@ xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
     GdkPixbuf *pix = NULL;
+    gint icon_size = width > height ? width : height;
 
-    pix = wnck_window_get_icon(window_icon->priv->window);
+    pix = xfw_window_get_icon(window_icon->priv->window, icon_size, 1);
     if(pix) {
         if(gdk_pixbuf_get_height(pix) != height) {
             pix = gdk_pixbuf_scale_simple(pix, height, height, GDK_INTERP_BILINEAR);
@@ -161,7 +161,7 @@ xfdesktop_window_icon_peek_label(XfdesktopIcon *icon)
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
 
     if(!window_icon->priv->label)
-        window_icon->priv->label = g_strdup(wnck_window_get_name(window_icon->priv->window));
+        window_icon->priv->label = g_strdup(xfw_window_get_name(window_icon->priv->window));
 
     return window_icon->priv->label;
 }
@@ -177,8 +177,9 @@ xfdesktop_window_icon_activated(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
 
-    wnck_window_activate(window_icon->priv->window,
-                         gtk_get_current_event_time());
+    xfw_window_activate(window_icon->priv->window,
+                        gtk_get_current_event_time(),
+                        NULL);
 
     return TRUE;
 }
@@ -188,13 +189,14 @@ xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
                                             GtkWidget *menu)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    GtkWidget *amenu = wnck_action_menu_new(window_icon->priv->window);
-    GtkWidget *mi;
+    GtkWidget *amenu = xfw_window_action_menu_new(window_icon->priv->window);
+    GtkWidget *mi, *img;
 
-    mi = gtk_menu_item_new_with_mnemonic(_("_Window Actions"));
+    img = gtk_image_new_from_icon_name("", GTK_ICON_SIZE_MENU);
+    mi = xfdesktop_menu_create_menu_item_with_mnemonic(_("_Window Actions"), img);
     gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), amenu);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    gtk_widget_show_all(mi);
+    gtk_widget_show(mi);
 
     return TRUE;
 }
@@ -202,7 +204,7 @@ xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
 
 
 XfdesktopWindowIcon *
-xfdesktop_window_icon_new(WnckWindow *window,
+xfdesktop_window_icon_new(XfwWindow *window,
                           gint workspace)
 {
     XfdesktopWindowIcon *icon = g_object_new(XFDESKTOP_TYPE_WINDOW_ICON, NULL);
